@@ -22,12 +22,13 @@
 
 #include <config.h>
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <bonobo/bonobo-dock.h>
-#include <bonobo/bonobo-dock-band.h>
-#include <bonobo/bonobo-dock-item.h>
+#include "bonobo-dock.h"
+#include "bonobo-dock-band.h"
+#include "bonobo-dock-item.h"
 
-
+G_DEFINE_TYPE (BonoboDockBand, bonobo_dock_band, GTK_TYPE_CONTAINER)
 
 #define noBONOBO_DOCK_BAND_DEBUG
 
@@ -45,22 +46,6 @@
 #define DEBUG(x)
 #endif
 
-
-
-struct _BonoboDockBandPrivate
-{
-	int dummy;
-	/* Nothing right now, needs to get filled with the private things */
-	/* XXX: When stuff is added, uncomment the allocation in the
-	 * bonobo_dock_band_init function! */
-};
-
-
-
-static void     bonobo_dock_band_class_init    (BonoboDockBandClass *class);
-
-static void     bonobo_dock_band_init          (BonoboDockBand *app);
-
 static void     bonobo_dock_band_size_request  (GtkWidget *widget,
                                                GtkRequisition *requisition);
 
@@ -81,7 +66,6 @@ static void     bonobo_dock_band_forall        (GtkContainer *container,
                                                GtkCallback callback,
                                                gpointer callback_data);
 
-static void     bonobo_dock_band_destroy       (GtkObject *object);
 static void     bonobo_dock_band_finalize      (GObject *object);
 
 static void     size_allocate_child           (BonoboDockBand *band,
@@ -140,28 +124,18 @@ static gboolean dock_empty_right              (BonoboDockBand *band,
 static gboolean check_guint_arg               (GObject *object,
 					       const gchar *name,
 					       guint *value_return);
-
-
-static GtkContainerClass *parent_class = NULL;
-
-
 
 static void
-bonobo_dock_band_class_init (BonoboDockBandClass *class)
+bonobo_dock_band_class_init (BonoboDockBandClass *klass)
 {
-  GtkObjectClass *object_class;
   GObjectClass *gobject_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
 
-  object_class = (GtkObjectClass *) class;
-  gobject_class = (GObjectClass *) class;
-  widget_class = (GtkWidgetClass *) class;
-  container_class = (GtkContainerClass *) class;
+  gobject_class = (GObjectClass *) klass;
+  widget_class = (GtkWidgetClass *) klass;
+  container_class = (GtkContainerClass *) klass;
 
-  parent_class = gtk_type_class (gtk_container_get_type ());
-
-  object_class->destroy = bonobo_dock_band_destroy;
   gobject_class->finalize = bonobo_dock_band_finalize;
 
   widget_class->map = bonobo_dock_band_map;
@@ -177,13 +151,11 @@ bonobo_dock_band_class_init (BonoboDockBandClass *class)
 static void
 bonobo_dock_band_init (BonoboDockBand *band)
 {
+  GtkWidget *widget = GTK_WIDGET (band);
+
   GTK_WIDGET_SET_FLAGS (band, GTK_NO_WINDOW);
 
   band->_priv = NULL;
-  /* XXX: when there is some private stuff enable this
-  band->_priv = g_new0(BonoboDockBandPrivate, 1);
-  */
-
   band->orientation = GTK_ORIENTATION_HORIZONTAL;
 
   band->children = NULL;
@@ -200,6 +172,9 @@ bonobo_dock_band_init (BonoboDockBand *band)
   band->drag_allocation.width = band->drag_allocation.height = 0;
 
   band->new_for_drag = FALSE;
+
+  if (GTK_WIDGET_VISIBLE (widget))
+    gtk_widget_queue_resize (widget);
 }
 
 
@@ -247,7 +222,7 @@ bonobo_dock_band_size_request (GtkWidget *widget,
 						     &preferred_width);
 
 	      if (has_preferred_width)
-		c->max_space_requisition = MAX (preferred_width, req.width);
+		c->max_space_requisition = MAX ((int)preferred_width, req.width);
 	      else
 		c->max_space_requisition = req.width;
 	    }
@@ -261,7 +236,7 @@ bonobo_dock_band_size_request (GtkWidget *widget,
 						      &preferred_height);
 
 	      if (has_preferred_height)
-		c->max_space_requisition = MAX (preferred_height, req.height);
+		c->max_space_requisition = MAX ((int)preferred_height, req.height);
 	      else
 		c->max_space_requisition = req.height;
 	    }
@@ -563,8 +538,7 @@ bonobo_dock_band_map (GtkWidget *widget)
   g_return_if_fail(widget != NULL);
   g_return_if_fail(BONOBO_IS_DOCK_BAND(widget));
 
-  if (GTK_WIDGET_CLASS (parent_class)->map != NULL)
-    (* GTK_WIDGET_CLASS (parent_class)->map) (widget);
+  GTK_WIDGET_CLASS (bonobo_dock_band_parent_class)->map (widget);
 
   for (lp = band->children; lp != NULL; lp = lp->next)
     {
@@ -585,8 +559,7 @@ bonobo_dock_band_unmap (GtkWidget *widget)
   g_return_if_fail(widget != NULL);
   g_return_if_fail(BONOBO_IS_DOCK_BAND(widget));
 
-  if (GTK_WIDGET_CLASS (parent_class)->unmap != NULL)
-    (* GTK_WIDGET_CLASS (parent_class)->unmap) (widget);
+  GTK_WIDGET_CLASS (bonobo_dock_band_parent_class)->unmap (widget);
 
   for (lp = band->children; lp != NULL; lp = lp->next)
     {
@@ -622,12 +595,10 @@ bonobo_dock_band_remove (GtkContainer *container, GtkWidget *widget)
   child = find_child (band, widget);
   if (child != NULL)
     {
-      gboolean was_visible;
 
       if (child == band->floating_child)
         band->floating_child = NULL;
 
-      was_visible = GTK_WIDGET_VISIBLE (widget);
       gtk_widget_unparent (widget);
 
       band->children = g_list_remove_link (band->children, child);
@@ -675,14 +646,6 @@ bonobo_dock_band_forall (GtkContainer *container,
     }
 }
 
-static void
-bonobo_dock_band_destroy (GtkObject *object)
-{
-  /* remember, destroy can be run multiple times! */
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
 static void
 bonobo_dock_band_finalize (GObject *object)
 {
@@ -691,8 +654,7 @@ bonobo_dock_band_finalize (GObject *object)
   g_free (self->_priv);
   self->_priv = NULL;
 
-  if (G_OBJECT_CLASS (parent_class)->finalize)
-    (* G_OBJECT_CLASS (parent_class)->finalize) (object);
+  G_OBJECT_CLASS (bonobo_dock_band_parent_class)->finalize (object);
 }
 
 
@@ -1044,8 +1006,7 @@ dock_nonempty (BonoboDockBand *band,
   GtkOrientation orig_item_orientation;
   GtkRequisition item_requisition;
   GList *lp, *next;
-  guint amount;
-  guint requirement;
+  gint amount, requirement;
 
   DEBUG (("entering function"));
 
@@ -1070,7 +1031,7 @@ dock_nonempty (BonoboDockBand *band,
   else
     requirement = item_requisition.height;
 
-  if (c->drag_prev_space + c->drag_foll_space < requirement)
+  if ((c->drag_prev_space + c->drag_foll_space) < requirement)
     {
       DEBUG (("not enough space %d %d",
               c->drag_prev_space + c->drag_foll_space,
@@ -1372,8 +1333,8 @@ dock_empty_right (BonoboDockBand *band,
     return FALSE;
 
   bonobo_dock_item_handle_size_request (item, &item_requisition);
-  if (c->drag_prev_space + c->drag_foll_space
-      < (guint) (band->orientation == GTK_ORIENTATION_HORIZONTAL
+  if ((c->drag_prev_space + c->drag_foll_space)
+      < (band->orientation == GTK_ORIENTATION_HORIZONTAL
                  ? item_requisition.width
                  : item_requisition.height))
     {
@@ -1446,6 +1407,8 @@ check_guint_arg (GObject *object,
 {
   GParamSpec *pspec;
 
+  g_return_val_if_fail (object != NULL, FALSE);
+
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (object), name);
   if (pspec != NULL) {
     GValue value = { 0, };
@@ -1475,40 +1438,9 @@ GtkWidget *
 bonobo_dock_band_new (void)
 {
   BonoboDockBand *band;
-  GtkWidget *widget;
 
-  band = gtk_type_new (bonobo_dock_band_get_type ());
-  widget = GTK_WIDGET (band);
-
-  if (GTK_WIDGET_VISIBLE (widget))
-    gtk_widget_queue_resize (widget);
-
-  return widget;
-}
-
-GtkType
-bonobo_dock_band_get_type (void)
-{
-  static GtkType band_type = 0;
-
-  if (band_type == 0)
-    {
-      GtkTypeInfo band_info =
-      {
-	"BonoboDockBand",
-	sizeof (BonoboDockBand),
-	sizeof (BonoboDockBandClass),
-	(GtkClassInitFunc) bonobo_dock_band_class_init,
-	(GtkObjectInitFunc) bonobo_dock_band_init,
-        /* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-	(GtkClassInitFunc) NULL,
-      };
-
-      band_type = gtk_type_unique (gtk_container_get_type (), &band_info);
-    }
-
-  return band_type;
+  band = g_object_new (bonobo_dock_band_get_type (), NULL);
+  return GTK_WIDGET (band);
 }
 
 /**
@@ -1569,6 +1501,11 @@ bonobo_dock_band_insert (BonoboDockBand *band,
       && !docking_allowed (band, BONOBO_DOCK_ITEM (child)))
     return FALSE;
 
+  if (BONOBO_IS_DOCK_ITEM (child) &&
+      !bonobo_dock_item_set_orientation (BONOBO_DOCK_ITEM (child),
+					 band->orientation))
+      return FALSE;
+
   if (position < 0 || position > (gint) band->num_children)
     position = band->num_children;
 
@@ -1586,14 +1523,9 @@ bonobo_dock_band_insert (BonoboDockBand *band,
       GList *p;
 
       p = g_list_nth (band->children, position);
-      g_list_prepend (p, band_child);
+      p = g_list_prepend (p, band_child);
     }
 
-  if (BONOBO_IS_DOCK_ITEM (child)
-      && ! bonobo_dock_item_set_orientation (BONOBO_DOCK_ITEM (child),
-                                            band->orientation))
-      return FALSE;
-
   gtk_widget_set_parent (child, GTK_WIDGET (band));
 
   if (GTK_WIDGET_REALIZED (child->parent))
@@ -1970,3 +1902,78 @@ bonobo_dock_band_layout_add (BonoboDockBand *band,
                                     child_num, child->offset);
     }
 }
+
+static BonoboDock *
+get_dock (GtkWidget *widget)
+{
+	while (widget && !BONOBO_IS_DOCK (widget))
+		widget = widget->parent;
+
+	return (BonoboDock *) widget;
+}
+
+gint
+_bonobo_dock_band_handle_key_nav (BonoboDockBand *band,
+				  BonoboDockItem *item,
+				  GdkEventKey    *event)
+{
+  gboolean handled = FALSE;
+
+  g_return_val_if_fail (BONOBO_IS_DOCK_BAND (band), FALSE);
+  g_return_val_if_fail (BONOBO_IS_DOCK_ITEM (item), FALSE);
+
+  if (event->state & GDK_CONTROL_MASK)
+    {
+      GList *l;
+      int cur_idx = 0;
+      int dest_idx;
+      int num_children = g_list_length (band->children);
+
+      for (l = band->children; l; l = l->next)
+        {
+          BonoboDockBandChild *child = l->data;
+          if (child->widget == (GtkWidget *)item)
+            break;
+          cur_idx++;
+	}
+
+      g_return_val_if_fail (l != NULL, FALSE);
+      
+      dest_idx = cur_idx;
+      if (band->orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+	  if (event->keyval == GDK_Left)
+
+	      dest_idx--;
+	  if (event->keyval == GDK_Right)
+	      dest_idx++;
+	}
+      else
+        {
+	  if (event->keyval == GDK_Up)
+	      dest_idx--;
+	  if (event->keyval == GDK_Down)
+	      dest_idx++;
+	}
+
+      if (dest_idx >= num_children)
+	  dest_idx = num_children - 1;
+      if (dest_idx < 0)
+	  dest_idx = 0;
+      if (dest_idx != cur_idx)
+        {
+          handled = TRUE;
+	  bonobo_dock_band_move_child (band, l, dest_idx);
+	}
+  }
+
+  if (!handled)
+    {
+      BonoboDock *dock = get_dock (GTK_WIDGET (band));
+
+      if (dock)
+        handled = _bonobo_dock_handle_key_nav (dock, band, item, event);
+    }
+
+  return handled;
+}
